(function(window) {
var Cube3D = function(x,y,z,w,h,d, color, data)
{
	this.initialize(x,y,z,w,h,d, color, data);
}
var p = Cube3D.prototype = new Object3D();

p.initialize = function(x,y,z,w,h,d, color, data)
{
	Object3D.prototype.initialize.apply(this);
	//x,y,z is center point on floor plane
	this.x = x;
	this.y = y;
	this.z = z;
	this.w = w;
	this.h = h;
	this.d = d;
	this.setColor(color);
	//data can be anything, for debugging mainly
	this.data = data;
	
	//precalc the 8 points describing the cube:
	//LBN,RBN,RBF,LBF,LTN,RTN,RTF,LTF (Left/Right Bottom/Top Near/Far)
	var hw = w/2;
	var hd = d/2;
	this.points = [];
	this.points[Corner.LBN] =	vec3.fromValues(x-hw,y,z+hd);
	this.points[Corner.RBN] =	vec3.fromValues(x+hw,y,z+hd);
	this.points[Corner.RBF] =	vec3.fromValues(x+hw,y,z-hd);
	this.points[Corner.LBF] =	vec3.fromValues(x-hw,y,z-hd);
	this.points[Corner.LTN] =	vec3.fromValues(x-hw,y+h,z+hd);
	this.points[Corner.RTN] =	vec3.fromValues(x+hw,y+h,z+hd);
	this.points[Corner.RTF] =	vec3.fromValues(x+hw,y+h,z-hd);
	this.points[Corner.LTF] =	vec3.fromValues(x-hw,y+h,z-hd);
	//define faces with refs to transformed points
	var pt = this.points;
	this.faces = [];
	this.faces2 = [];
	//
	this.faces[Face.T] = {points:[pt[Corner.LTN],pt[Corner.RTN],pt[Corner.RTF],pt[Corner.LTF]]};
	this.faces[Face.L] = {points:[pt[Corner.LBN],pt[Corner.LBF],pt[Corner.LTF],pt[Corner.LTN]]};
	this.faces[Face.N] = {points:[pt[Corner.LBN],pt[Corner.RBN],pt[Corner.RTN],pt[Corner.LTN]]};
	this.faces[Face.R] = {points:[pt[Corner.RBN],pt[Corner.RBF],pt[Corner.RTF],pt[Corner.RTN]]};
	this.faces[Face.F] = {points:[pt[Corner.LBF],pt[Corner.RBF],pt[Corner.RTF],pt[Corner.LTF]]};
	//store rectangle of ground for quick 2d collision detection
	this.groundRect = new Rectangle(x-hw, z-hd, w, d);
	//some stored values for z ordering
	this.xMin = this.points[Corner.LBF][0];
	this.xMax = this.points[Corner.RBN][0];
	this.zMin = this.points[Corner.LBF][2];
	this.zMax = this.points[Corner.RBN][2];
	//log(this.xMin,this.xMax,this.zMin,this.zMax);
	//some reusable vectors:
	this.v3_0 = vec3.create();
	this.v3_1 = vec3.create();
	this.v3_2 = vec3.create();
}

p.setColor = function(color)
{
	this.color = color;
	this.colorShadow = ColorUtil.interpolateHex(this.color,"#000000", Config.shadowStrength);
}
p.setHeight = function(h)
{
	this.h = h;
	//
	this.points[Corner.LTN][1] = this.y + h;
	this.points[Corner.RTN][1] = this.y + h;
	this.points[Corner.RTF][1] = this.y + h;
	this.points[Corner.LTF][1] = this.y + h;
}

p.applySceneTransform = function(sceneTransformMatrix)
{
	//check culling of faces
	//Normally dot product of surface normal and camera-to-face vector determines culling
	//But we have a fixed cam rotation and axis aligned faces, so we can take a shortcut
	//Works only for 90deg x-rotated camera
	var p0 = this.v3_0, p1 = this.v3_1, p2 = this.v3_2;//reuse vec3 objects
	var ps = this.points;
	vec3.transformMat4(p0, ps[0], sceneTransformMatrix);//LBN
	vec3.transformMat4(p1, ps[1], sceneTransformMatrix);//RBN
	vec3.transformMat4(p2, ps[2], sceneTransformMatrix);//RBF
	//
	this.faces2.length = 0;
	if (0<p0[0]) this.faces2[Face.L] = this.faces[Face.L];
	else if (0>p1[0]) this.faces2[Face.R] = this.faces[Face.R];
	//NB: check y because of rotationX of scene
	if (0>p0[1]) this.faces2[Face.N] = this.faces[Face.N];
	else if (0<p2[1]) this.faces2[Face.F] = this.faces[Face.F];
	this.faces2[Face.T] = this.faces[Face.T];//top
	//cache dist to camera position for zordering
	vec3.positionFromMat4(p0, sceneTransformMatrix);//scene origin
	//NB: this.x+p0[0] and not -, because cam pos is inverse of scene position!
	this.dist = Math.pow(this.x+p0[0],2) + Math.pow(this.z+p0[1],2);
	this.distX = Math.abs(this.x+p0[0]);
	this.distZ = Math.abs(this.z+p0[1]);
	
	//NB: we use p0[1] and not p0[2] because of scene rotationX
}

p.getFaces = function()
{
	return this.faces2;
}
p.getFace = function(face, unculled)
{
	if (unculled) return this.faces[face];
	return this.faces2[face];
}


window.Cube3D = Cube3D;
}(window));