(function(window) {
	var Piece = function(canvas, config) {
		this.initialize(canvas, config);
	}
	var p = Piece.prototype = new BasePiece();
	p.paused = false;
	
	p.initialize = function(canvas, config)
	{
		if (isMobile) config.framerate = config.framerateMobile;
		BasePiece.prototype.initialize.apply(this, [canvas, config]);
		//
		this.bg = new Shape();
		this.addChild(this.bg);
		//
		this.viewportMargin = 1.6;//for generator viewport, should be higher for greather max height of buildings!
		this.init3d();
		this.initInteraction();
		//
		this.setMode(Mode.NORMAL);
	}
	
	p.setSize = function(w,h, dpr)
	{
		w *= dpr;
		h *= dpr;
		this.x = w/2;
		this.y = h/2;
		this.width = w;
		this.height = h;
		this.area = w*h;
		this.diam = Math.sqrt(this.area);
		this.scene.setSize(w,h);
		//
		this.dragThrowFactor = 0;
		this.dragSpeed = 0;
		//
		this.drawBG();
		if (!this.scene.objects.length) this.reset();
		else this.scheduleReset();
		//
		this.stage.update();
	}
	
	p.drawBG = function()
	{
		var cfg = this.config;
		var c = this.mode==Mode.NORMAL ? cfg.groundColor : cfg.groundColorOutlineMode;
		this.bg.graphics.clear().beginFill(c).drawRect(-this.x,-this.y,this.width,this.height);
	}
	
	p.onKeyUp = function(e)
	{
		BasePiece.prototype.onKeyUp.apply(this, [e]);
		if (!this.config.debug) return;
		var c = String.fromCharCode(e.which);
		if (c=="R") this.reset();
		else if (c=="M") this.setMode(1-this.mode);
		else if (c=="G") this.config.showGrid = !this.config.showGrid;
		else if (c=="C") log(this.scene.objects.length);
		else if (c=="Z")
		{
			this.scene.showFloorPlan = !this.scene.showFloorPlan;
			this.scene.render();
			this.stage.update();
		}
	}
	
	p.init3d = function()
	{
		this.mapRectOnObject = {};
		this.scene = new Scene3D(this.canvas.width, this.canvas.height, this.config.focalLength, this.config.lightVector);
		this.addChild(this.scene);
		//
		var that = this;
		this.viewportScene = new Rectangle();
		this.viewportGenerator = new Rectangle();
		this.viewportGenerator2 = new Rectangle();
		this.generator = new Generator(
			this.config.tileDefs, this.config.tileSizeFactor * (isMobile?this.config.mobileScaleFactor:1),
			function(r) { return that.onGeneratorRectAdded(r); },
			function(r) { return that.onGeneratorRectRemoved(r); }
		);
		this.scene.mouseChildren = this.scene.mouseEnabled = false;
	}
	
	p.onGeneratorRectAdded = function(r)
	{
		var color = RandomUtil.pick(this.config.colors);
		var data = r.id + "\n"+ [r.x,r.y,r.width,r.height].join(",");
		//var height = Math.floor(RandomUtil.between(this.config.heightMin,this.config.heightMax) * Math.sqrt(this.area)/25);
		var p = Math.random();
		var f = (p < .5) ? p * 2 : (1 - p) * 2;
		f = Math.pow(f, this.config.heightExtremeFactor);
		var p2 = (p < .5) ? f / 2 : 1 - (f / 2);
		//log(p,p2);
		var height = this.config.heightMin + p2 * (this.config.heightMax-this.config.heightMin);
		height *= Math.sqrt(this.area)/25;
		//add margins to all sides of rect
		var ref = (this.width+this.height)/2;
		var mr = Math.round(RandomUtil.between(this.config.marginMin,this.config.marginMax) * ref);
		var ml = Math.round(RandomUtil.between(this.config.marginMin,this.config.marginMax) * ref);
		var mt = Math.round(RandomUtil.between(this.config.marginMin,this.config.marginMax) * ref);
		var mb = Math.round(RandomUtil.between(this.config.marginMin,this.config.marginMax) * ref);
		//
		//log("onGeneratorRectAdded",height, color, r.toString(),mr,ml,mt,mb);
		var w = r.width-mr-ml;
		var d = r.height-mt-mb;
		var cube = new Cube3D(r.x+ml+w/2, 0, r.y+mt+d/2, w, height, d, color, data);
		//link r.id to cube for later reference (removal)
		this.mapRectOnObject[r.id] = cube;
		this.scene.addObject(cube);
	}
	p.onGeneratorRectRemoved = function(r)
	{
		var cube = this.mapRectOnObject[r.id];
		this.scene.removeObject(cube);
		delete this.mapRectOnObject[r.id];
	}
		
	p.initInteraction = function()
	{
		this.interaction = new Interaction(this);
	}
	p.startDrag = function(x,y)
	{
		this.dragged = false;
		this.dragX = x;
		this.dragY = y;
		this.dragging = true;
		//
		if (this.config.toggleModeAction=="down") this.toggleMode();
	}
	p.stopDrag = function()
	{
		this.dragging = false;
		if (this.dragged)
		{
			this.dragSpeed = Math.sqrt(this.dx*this.dx + this.dy*this.dy);
			this.dragAngle = this.autoMoveAngle = Math.atan2(this.dy, this.dx);
			this.dragThrowFactor = 1;
			this.toggledAfterBrake = false;
		}
		else
		{
			this.startAutoMove();
			this.dragThrowFactor = null;
		}
		if (this.config.toggleModeAction=="up") this.toggleMode();
		else if (this.config.toggleModeAction=="click" && !this.dragged) this.toggleMode();
	}
	p.updateDrag = function(x,y)
	{
		if (!this.dragging) return;
		this.dx = x-this.dragX;
		this.dy = y-this.dragY;
		this.offsetX += this.dx;
		this.offsetY += this.dy;
		this.dragX = x;
		this.dragY = y;
		this.changed = true;
		this.dragged = true;
	}
	
	p.startAutoMove = function()
	{
		this.autoMoveAngle = null;
		switch (this.config.autoMoveDirection)
		{
			case "random":
				this.autoMoveAngle = Math.random()*Math.PI*2;
				break;
			case "last":
				this.autoMoveAngle = this.dragAngle!=null ? this.dragAngle : Math.random()*Math.PI*2;
				break;
			default:
				var deg = parseFloat(this.config.autoMoveDirection);
				if (!isNaN(deg)) this.autoMoveAngle = deg/180*Math.PI;
		}
	}
	p.stopAutoMove = function()
	{
		this.autoMoveAngle = null;
	}
	
	p.toggleMode = function(skipRender)
	{
		this.setMode(1-this.mode);
		if (!skipRender)
		{
			this.scene.render(this.viewportScene);
			this.stage.update();
		}
	}
	p.setMode = function(value)
	{
		if (this.mode==value) return;
		this.mode = value;
		this.scene.setMode(value);
		this.drawBG();
	}
	
	
	p.scheduleReset = function()
	{
		this.cancelReset();
		this.scene.reset();
		var that = this;
		this.timeoutReset = setTimeout(function() { that.reset(); }, 500);
	}
	p.cancelReset = function()
	{
		if (this.timeoutReset) clearTimeout(this.timeoutReset);
		this.timeoutReset = null;
	}
	p.reset = function()
	{
		this.cancelReset();
		this.mapRectOnObject = {};
		this.scene.reset();
		this.offsetX = 0;
		this.offsetY = 0;
		var m = this.viewportMargin;
		var w = this.width*m;
		var h = this.height*m;
		this.viewportGenerator.set(-w/2,-h/2,w,h);
		this.generator.generate(this.viewportGenerator);
		this.changed = true;
	}
	p.start = function()
	{
		BasePiece.prototype.start.apply(this);
		if (this.config.colorChangeOn) this.scheduleNextColorChange();
		this.startAutoMove();
	}
	
	p.scheduleNextColorChange = function()
	{
		var d = RandomUtil.between(this.config.colorChangeIntervalMin,this.config.colorChangeIntervalMax);
		this.nextColorChange = Date.now() + d;
	}
	p.checkColorChange = function()
	{
		if (Date.now()>=this.nextColorChange)
		{
			if (this.mode==Mode.NORMAL)
			{
				var os = this.scene.objectsVisible;
				for (var i=0;i<os.length;i++)
				{
					var o = os[i];
					if (!o) return;
					var color = RandomUtil.pick(this.config.colors,[o.color]);
					o.setColor(color);
				}
				this.changed = true;
			}
			this.scheduleNextColorChange();
		}
	}
	p.update = function()
	{
		if (this.config.colorChangeOn) this.checkColorChange();
		//check auto offset
		if (!this.dragging && this.autoMoveAngle!=null)
		{
			this.dragThrowFactor *= this.config.brakeFactor;
			var smin = 1000 * this.config.autoMoveSpeed / this.diam;
			var s = smin + Math.max(0, (this.dragSpeed - smin) * this.dragThrowFactor);
			//correct s for elapsed frametime:
			s *= this.config.framerate * this.tickDuration / 1000;
			var a = this.autoMoveAngle;
			this.offsetX += Math.cos(a) * s;
			this.offsetY += Math.sin(a) * s;
			this.changed = true;
			if (this.dragThrowFactor<.01 && this.config.autoMoveSpeed==0) this.stopAutoMove();
			if (!this.toggledAfterBrake && this.config.toggleModeAction=="still" && this.dragThrowFactor<.01)
			{
				this.toggledAfterBrake = true;
				this.toggleMode();
			}
		}
		//
		if (!this.changed || this.timeoutReset) return;
		this.changed = false;
		this.updateViewport();
		mat4.setXYZ(this.scene.matrix,this.offsetX,this.offsetY,0);
		this.scene.render(this.viewportScene);
		return true;
	}
	p.updateViewport = function()
	{
		//update scene viewport
		var w = this.width;
		var h = this.height;
		var vs = this.viewportScene;
		vs.set(-w/2-this.offsetX,-h/2-this.offsetY,w,h);
		//change viewport of generator when offset is big enough
		var m = this.viewportMargin;
		var vp = this.viewportGenerator;
		var vp2 = this.viewportGenerator2;
		w *= m;
		h *= m;
		vp2.set(-w/2-this.offsetX,-h/2-this.offsetY,w,h);
		if (Math.abs(vp2.x-vp.x)>w/(m*10) || Math.abs(vp2.y-vp.y)>h/(m*10))
		{
			vp.copy(vp2);
			this.generator.setViewport(vp);
		}
	}
	
	window.Piece = Piece;
}(window));