(function(window) {
var Shader = function(lightVector) {
	this.initialize(lightVector);
}

var p = Shader.prototype;

p.initialize = function(lightVector)
{
	this.lightVector = lightVector;//vec3.normalize(vec3.create(), lightVector);
	//cache some often used values:
	//this.castingPoints;//indices of corner points of casting obj involved in shading
	this.receivingFaces = {y:Face.T};//indices of faces of receiving obj involved in shading
	if (this.lightVector[0]>0)
	{
		this.receivingFaces.x = Face.L;
		if (this.lightVector[2]<0) this.castingPoints = [Corner.LBN,Corner.RBN,Corner.RTN,Corner.RTF,Corner.LTF,Corner.LBF];
		else this.castingPoints = [Corner.LBF,Corner.RBF,Corner.RTF,Corner.RTN,Corner.LTN,Corner.LBN];
	}
	else if (this.lightVector[0]<0)
	{
		this.receivingFaces.x = Face.R;
		if (this.lightVector[2]<0) this.castingPoints = [Corner.RBN,Corner.LBN,Corner.LTN,Corner.LTF,Corner.RTF,Corner.RBF];
		else this.castingPoints = [Corner.RBF,Corner.LBF,Corner.LTF,Corner.LTN,Corner.RTN,Corner.RBN];
	}
	if (this.lightVector[2]>0)
	{
		this.receivingFaces.z = Face.F;
	}
	else if (this.lightVector[2]<0)
	{
		this.receivingFaces.z = Face.N;
	}
	//objects for pooling:
	this.v0 = vec3.create();
	this.v1 = vec3.create();
	this.arr0 = [];
	this.arr1 = [];
}
//public
p.reset = function()
{
	this.objects = [];
	//stores for shadow information:
	this.groundShadows = {};//by obj id, pointlist describing shadow on ground
	this.groundShadowRects = {};//by obj id, rectangle bounding box of shadow on ground
	this.shadows = {};//by obj id
	//map to store for each obj on which obj it casts which shadow
	this.mapObjectOnPoly = {};//obj id on obj id on {poly,face}
}
//public
p.addObject = function(obj)
{
	//obj is Cube3D
	//add ground shadow
	this.calcGroundShadowForObj(obj);
	//add shadows from and on other objects
	this.calcShadowsForObject(obj);
	this.objects.push(obj);
}
//public
p.removeObject = function(obj)
{
	var idx = this.objects.indexOf(obj);
	if (idx>-1) this.objects.splice(idx,1);
	//we remove shadows on other objects as well
	var m = this.mapObjectOnPoly[obj.id];
	if (m)
	{
		for (var i=0;i<m.length;i++)
		{
			//During adding: this.shadows[obj1.id][faceIdx].push(poly);
			var data = m[i];//props: object (id), poly, face (index)
			if (this.shadows[data.object])
			{
				var arr = this.shadows[data.object][data.face];
				idx = arr.indexOf(data.poly);
				if (idx>-1) arr.splice(idx,1);
			}
		}
	}
	//remove shadows cast by other objects as well
	//TODO expensive but perhaps not very important, so skip for now
	delete this.groundShadows[obj.id];
	delete this.groundShadowRects[obj.id];
	delete this.shadows[obj.id];
	delete this.mapObjectOnPoly[obj.id];
}
//public
p.updateObject = function(obj)
{
	this.removeObject(obj);
	this.addObject(obj);
}

p.calcGroundShadowForObj = function(obj)
{
	//store bounding box of ground shadow for quick 2d calc
	this.groundShadowRects[obj.id] = obj.groundRect.clone();//Rectangle
	var ray = this.v0;
	//calc polygon of light on floor (y=0)
	var ps = this.castingPoints;//ps contains points of top plane that are involved in shadow
	this.groundShadows[obj.id] = [];
	for (var i=0;i<ps.length;i++)
	{
		var p = obj.points[ps[i]];
		vec3.scale(ray, this.lightVector, -p[1] / this.lightVector[1]);
		var p1 = vec3.add(vec3.create(), p, ray);
		this.groundShadows[obj.id].push(p1);
		this.groundShadowRects[obj.id].unionXY(p1[0],p1[2]);
	}
}

p.calcShadowsForObject = function(obj)
{
	var len = this.objects.length;
	for (var i=0;i<len;i++)
	{
		var obj1 = this.objects[i];
		if (obj==obj1) return;
		//first do quick test to see if objs might cast shadows on eachother
		//log(obj.id, obj1.id);
		//log(obj.id, obj.groundRect.toString(), this.groundShadowRects[obj.id].toString());
		//log(obj1.id, obj1.groundRect.toString(), this.groundShadowRects[obj1.id].toString());
		//log(this.groundShadowRects[obj.id].intersects(obj1.groundRect), this.groundShadowRects[obj1.id].intersects(obj.groundRect));
		if (this.groundShadowRects[obj.id].intersects(obj1.groundRect)) this.castShadowFromObjOnObj(obj,obj1);
		else if (this.groundShadowRects[obj1.id].intersects(obj.groundRect)) this.castShadowFromObjOnObj(obj1,obj);
	}
}

p.castShadowFromObjOnObj = function(obj0,obj1)
{
	//log("Shader.castShadowFromObjOnObj",obj0.id, obj1.id);
	var v = this.lightVector;
	var ps = this.castingPoints;
	//obj1 is in ground shadow rect of obj0 so there MIGHT be a shadow of obj0 on obj1
	//determine what faces of obj1 are intersected by light rays from corners of obj0
	//  but only faces facing the light!
	//For each receivingFace, cast ray from each castingPoints on plane of face,
	//  forming a shadow polygon on that plane.
	//  Then calc intersection of face with polygon
	var ray = this.v0;
	var paramsets = [["x",0,1,2],["y",1,2,0],["z",2,0,1]];
	//
	for (var m=0;m<paramsets.length;m++)
	{
		var params = paramsets[m];
		var prop = params[0];
		var dir = params[1];
		var dir1 = params[2];
		var dir2 = params[3];
		var faceIdx = this.receivingFaces[prop];
		var face = obj1.faces[faceIdx].points;
		var v = face[0][dir];//value for x/y/z for face plane
		//log(prop,v);
		var poly2d = ArrayUtil.clear(this.arr0);//store in temp 2d poly (on plane of face)
		var p2 = this.v1;
		for (var i=0;i<ps.length;i++)
		{
			//calc intersection of rays through casting points with face plane
			//and store as poly
			var p = obj0.points[ps[i]];
			vec3.scale(ray, this.lightVector, (v-p[dir]) / this.lightVector[dir]);
			vec3.add(p2, p, ray);
			poly2d.push([p2[dir1],p2[dir2]]);//NB: 2d, for clipping method
		}
		//create temp 2d version of face
		var face2d = ArrayUtil.clear(this.arr1);
		for (i=0;i<face.length;i++) face2d.push([face[i][dir1],face[i][dir2]]);
		//calc intersection of face with polygon (clip)
		var clip = GeomUtil.clipPolygon(poly2d,face2d);
		//log(poly2d.join(","),face2d.join(","),clip.join(",") );
		if (clip.length)
		{
			//convert clip to 3d
			var poly = [];
			//order of dir/dir1/dir2 is diff for x y and z planes
			if (dir==0)
			{
				for (i=0;i<clip.length;i++) poly.push( vec3.fromValues(v,clip[i][0],clip[i][1]) );
			}
			else if (dir==2)
			{
				for (i=0;i<clip.length;i++) poly.push( vec3.fromValues(clip[i][0],clip[i][1],v) );
			}
			else
			{
				for (i=0;i<clip.length;i++) poly.push( vec3.fromValues(clip[i][1],v,clip[i][0]) );
			}
			//OPTIMIZE: create union with existing polys for better drawing performance?
			//	NB: after union there still might be multiple, unconnected polys!
			if (!this.shadows[obj1.id]) this.shadows[obj1.id] = [[],[],[],[]];
			if (!this.shadows[obj1.id][faceIdx]) this.shadows[obj1.id][faceIdx] = [];
			this.shadows[obj1.id][faceIdx].push(poly);
			//also store in map so we can later remove this poly
			//OPTIMIZE: this map is a memory/gc heavy object!
			if (this.mapObjectOnPoly[obj0.id]==null) this.mapObjectOnPoly[obj0.id] = [];
			this.mapObjectOnPoly[obj0.id].push({object:obj1.id,face:faceIdx,poly:poly});
		}
	}
}

p.getShadowsForObjectFace = function(obj, face)
{
	return this.shadows[obj.id] ? this.shadows[obj.id][face] : null;
}
p.getGroundShadowForObject = function(obj)
{
	return this.groundShadows[obj.id];
}
p.getGroundShadowRectForObject = function(obj)
{
	return this.groundShadowRects[obj.id];
}


window.Shader = Shader;
}(window));