var ArrayUtil = {};

ArrayUtil.shuffle = function(array)
{
	var res = [];
	while (array.length)
	{
		res.push(array.splice(Math.floor(Math.random()*array.length),1)[0]);
	}
	return res;
}


ArrayUtil.lookup = function(table, x, interpolate)
{
	//'table' should be array of format [x0,y0, x1,y1, x2,y2 ...], with x0,x1,x2 increasing
	if (x<table[0]) return table[1];//x < x0: return y0
	for (var i=0;i<table.length-2;i+=2)
	{
		var xp = table[i];
		var xn = table[i+2];
		if (xp<=x && x<=xn)
		{
			var yp = table[i+1];
			var yn = table[i+3];
			if (interpolate)
			{
				return yp + (yn-yp) * (x-xp)/(xn-xp);
			}
			else
			{
				return table[i+1];//return low y
			}
		}
	}
	//not found: x must be larger than largest x in table
	return table[table.length-1];//x > xLast: return yLast
}

/**
 * Return array with no doubles
 **/
ArrayUtil.unique = function(values)
{
	var res = [];
	for (var i=0;i<values.length;i++)
	{
		if (res.indexOf(values[i])==-1) res.push(values[i]);
	}
	return res;
}

/**
 *
 **/
ArrayUtil.sortNum = function(values)
{
	values.sort(function(a,b) { return a>b ? 1 : -1; });
}

/**
 * ArrayUtil.max will return first INDEX of greatest numeric value in values
 * To get the max value, just use Math.max
 **/
ArrayUtil.max = function(values)
{
	return values.indexOf( Math.max.apply(null, values) );
}

/**
 * ArrayUtil.sum will return sum of numeric values in values
 **/
ArrayUtil.sum = function(values)
{
	var sum = 0;
	for (var i=0;i<values.length;i++) sum += values[i];
	return sum;
}
/**
 * ArrayUtil.average will return average of numeric values in values
 **/
ArrayUtil.average = function(values)
{
	return values.length ? ArrayUtil.sum(values)/values.length : 0;
}

ArrayUtil.BOTH = 0;
ArrayUtil.LOWER = 1;
ArrayUtil.HIGHER = 2;
/**
 * Find index of closest numeric value in array.
 * Use direction to search for closest lower or higher value
 * @param	array Array containing numeric values. Does not have to be sorted.
 * @param	value
 * @param	direction ArrayUtil.BOTH (default), ArrayUtil.LOWER or ArrayUtil.HIGHER
 * @return index of closest value. In case direction is LOWER or HIGHER, index can be -1! If array is length 0, index is -1.
 */
ArrayUtil.indexOfClosest = function(array, value, direction)
{
	if (!array) return -1;
	if (!direction) direction = 0;
	var d = Infinity;
	var index = -1;
	var i;
	if (direction == ArrayUtil.BOTH)
	{
		for (i = 0; i < array.length; i++)
		{
			if (Math.abs(array[i] - value) < d)
			{
				d = Math.abs(array[i] - value)
				index = i;
			}
		}
	}
	else if (direction == ArrayUtil.LOWER)
	{
		for (i = 0; i < array.length; i++)
		{
			if (array[i] < value && value - array[i] < d)
			{
				d = value - array[i];
				index = i;
			}
		}
	}
	else if (direction == ArrayUtil.HIGHER)
	{
		for (i = 0; i < array.length; i++)
		{
			if (array[i] > value && array[i] - value < d)
			{
				d = array[i] - value;
				index = i;
			}
		}
	}
	return index;
}

/**
 * ArrayUtil.clear will empty the array and return the array for chaining;
 **/
ArrayUtil.clear = function(array)
{
	array.length = 0;
	return array;
}