(function(window) {
	/**
	 * 20131121 showing stats on touch device in debug mode
	 * 20131108 added log call to tick method to work around android bug
	 * 20131105 changes for easeljs 0.7.0
	 * 20131105 stage regpoint offset for pixel perfect (cardinal) lines
	 **/
	var BasePiece = function()
	{
	}
	var p = BasePiece.prototype = new Container();
	p.canvas;
	p.stage;
	p.config;
	p.tickCount = 0;//elapsed ticks
	p.tickLast;//timestamp of last tick
	p.tickDuration;//duration in msecs of last tick
	p.paused = false;
	//
	p.initialize = function(canvas, config)
	{
		Container.prototype.initialize.apply(this);
		//
		this.config = config;
		this.canvas = canvas;
		this.stage = new Stage(canvas);
		this.stage.addChild(this);
		this.stage.regX = this.stage.regY = -.5;//for pixel perfect (cardinal) lines
		//
		var that = this;
		window.onkeyup = function(e) { that.onKeyUp(e); };
		//catch/block touch events to prevent def browser behaviour
		if (window.ontouchstart !== undefined)
		{
			canvas.addEventListener('touchmove', function(e) { that.onTouchMove(e); });
			canvas.addEventListener('touchstart', function(e) { that.onTouchStart(e); });
			canvas.addEventListener('touchend', function(e) { that.onTouchEnd(e); });
			if (this.config.debug) this.addStats();
		}
	}
	
	p.addStats = function()
	{
		//call this in child init if we want stats
		this.stats = new Stats();
		this.stats.domElement.style.position	= 'absolute';
		this.stats.domElement.style.bottom	= '0px';
		this.stats.domElement.style.zIndex	= '99';
		document.body.appendChild( this.stats.domElement );
	}
	p.removeStats = function()
	{
		if (this.stats)
		{
			document.body.removeChild( this.stats.domElement );
			this.stats = null;
		}
	}
	
	p.onTouchStart = function(e)
	{
		e.preventDefault();
	}
	p.onTouchEnd = function(e)
	{
		e.preventDefault();
	}
	p.onTouchMove = function(e)
	{
		e.preventDefault();
	}
	
	p.onKeyUp = function(e)
	{
		if (!this.config.debug) return;
		var c = String.fromCharCode(e.which);
		if (c=="P") this.paused = !this.paused;
		else if (c=="S" && this.update()) this.stage.update();
		else if (c=="F")
		{
			if (this.stats) this.removeStats();
			else this.addStats();
		}
	}
	
	p.setSize = function(w,h,dpr)
	{
		this.stage.update();
	}
	
	p.start = function()
	{
		Ticker.setFPS(this.config.framerate);
		//Ticker.addListener(this);
		if (Ticker.addListener)
		{
			Ticker.addListener(this);//easeljs 0.6.0
		}
		else
		{
			var that = this;
			Ticker.addEventListener("tick", function(e) { that.handleTick(e); });//easeljs 0.7.0
		}
		this.tickLast = Date.now();
		this.tickDuration = 1/this.config.framerate;
	}
	
	p.tick = function()
	{
		this.handleTick(null);
	}
	p.handleTick = function(e)
	{
		if (this.stats) this.stats.update();
		//
		var now = Date.now();
		this.tickDuration = now-this.tickLast;
		this.tickLast = now;
		//
		if (this.paused) return;
		//
		this.tickCount ++;
		var b = this.update();
		if (b) this.stage.update();
		//avoid android hangup bug by forcing DOM element update
		log2(this.tickCount%2 ? "&nbsp;" : "&nbsp;&nbsp;");
	}

	p.update = function()
	{
		//override
		return true;
	}
	
	window.BasePiece = BasePiece;
}(window));