//TODO: put extensions for different createjs prototypes in different js files?

createjs.Rectangle.prototype.set = function(x,y,w,h)
{
	this.initialize(x,y,w,h);
}
/**
 * Calculate union of rect with given rect
 * Expands rect to contain given rect, so this rect is changed!
 * Returns self for chaining
 **/
createjs.Rectangle.prototype.union = function(r)
{
	var xmax = Math.max(this.x+this.width,r.x+r.width);
	var ymax = Math.max(this.y+this.height,r.y+r.height);
	this.x = Math.min(this.x,r.x);
	this.y = Math.min(this.y,r.y);
	this.width = xmax-this.x;
	this.height = ymax-this.y;
	return this;
}
/**
 * Calculate union of rect with a point (=bounding box of rect and point)
 * Expands rect to contain point, so this rect is changed!
 * Returns self for chaining
 **/
createjs.Rectangle.prototype.unionPoint = function(p)
{
	return this.unionXY(p.x,p.y);
}
/**
 * Calculate union of rect with a x,y coord (=bounding box of rect and point)
 * Expands rect to contain coordinate, so this rect is changed!
 * Returns self for chaining
 **/
createjs.Rectangle.prototype.unionXY = function(x,y)
{
	var xmax = Math.max(this.x+this.width,x);
	var ymax = Math.max(this.y+this.height,y);
	this.x = Math.min(this.x,x);
	this.y = Math.min(this.y,y);
	this.width = xmax-this.x;
	this.height = ymax-this.y;
	return this;//for chaining
}

createjs.Rectangle.prototype.expand = function(dx,dy)
{
	//TODO: check width>=-2*dx etc?
	this.x -= dx;
	this.width += 2*dx;
	this.y -= dy;
	this.height += 2*dy;
}

createjs.Rectangle.prototype.containsPoint = function(p)
{
	return p.x>=this.x && p.x<=this.x+this.width && p.y>this.y && p.y<=this.y+this.height;
}
createjs.Rectangle.prototype.contains = function(x,y)
{
	return x>=this.x && x<=this.x+this.width && y>this.y && y<=this.y+this.height;
}
createjs.Rectangle.prototype.containsRect = function(r)
{
	return this.x<=r.x && r.x+r.width<=this.x+this.width && this.y<=r.y && r.y+r.height<=this.y+this.height;
}
/**
 * Does this rectangle intersects specified rectangle?
 * NB: Containment is also considered intersection in this implementation!
 */
createjs.Rectangle.prototype.intersects = function(r)
{
	return !(this.x+this.width<r.x || r.x+r.width<this.x || this.y+this.height<r.y || r.y+r.height<this.y);
}


createjs.Graphics.prototype.dashedLine = function(x1, y1, x2, y2, dashLen) {
    this.moveTo(x1, y1);

    var dX = x2 - x1;
    var dY = y2 - y1;
    var dashes = Math.floor(Math.sqrt(dX * dX + dY * dY) / dashLen);
    var dashX = dX / dashes;
    var dashY = dY / dashes;

    var q = 0;
    while (q++ < dashes) {
        x1 += dashX;
        y1 += dashY;
        this[q % 2 == 0 ? 'moveTo' : 'lineTo'](x1, y1);
    }
    this[q % 2 == 0 ? 'moveTo' : 'lineTo'](x2, y2);
}
createjs.Graphics.prototype.drawDashedRect = function(x1, y1, w, h, dashLen) {
    this.moveTo(x1, y1);
    var x2 = x1 + w;
    var y2 = y1 + h;
    this.dashedLine(x1, y1, x2, y1, dashLen);
    this.dashedLine(x2, y1, x2, y2, dashLen);
    this.dashedLine(x2, y2, x1, y2, dashLen);
    this.dashedLine(x1, x2, x1, y1, dashLen);
}