var GeomUtil = {};

GeomUtil.polarToCartesian = function(length, angle)
{
	return new Point(Math.cos(angle) * length, Math.sin(angle) * length);
}

GeomUtil.distance = function(a, b)
{
    var dx = a.x - b.x;
    var dy = a.y - b.y;
    return Math.sqrt(dx*dx + dy*dy);
}

GeomUtil.getTriangleArea = function(p0,p1,p2)
{
	//heron's formula
	var l0 = GeomUtil.distance(p0,p1);
	var l1 = GeomUtil.distance(p1,p2);
	var l2 = GeomUtil.distance(p2,p0);
	var s = (l0+l1+l2)/2;//half perimeter
	var area = Math.sqrt(s * (s-l0) * (s-l1) * (s-l2));
	return area;
}

GeomUtil.getTriangleIncenter = function(p0,p1,p2)
{
	//http://en.wikipedia.org/wiki/Incenter#Coordinates_of_the_incenter
	//weighted average of the coordinates of the three vertices using the opposite side lengths of the triangle as weights
	var l0 = GeomUtil.distance(p0,p1);
	var l1 = GeomUtil.distance(p1,p2);
	var l2 = GeomUtil.distance(p2,p0);
	var x = (l1 * p0.x + l2 * p1.x + l0 * p2.x) / (l0 + l1 + l2);
	var y = (l1 * p0.y + l2 * p1.y + l0 * p2.y) / (l0 + l1 + l2);
	return new Point(x,y);
}

GeomUtil.Segment = function(a, b)
{
	//a,b are points / objects with props x,y
	var ab = {x: b.x - a.x, y: b.y - a.y};
	var length = GeomUtil.distance(a, b);
	
	this.distanceToPoint = function(p)
	{
		var cross = ab.x * (p.y-a.y) - ab.y * (p.x-a.x);
		return Math.min(GeomUtil.distance(a,p), GeomUtil.distance(b,p), Math.abs(cross / length));
	};
}

/**
 * Tests whether point p is contained in polygon
 * Polygon is defined as array of points (with x,y props)
 **/
GeomUtil.pointInPolygon = function(point, polygon)
{
	//http://stackoverflow.com/questions/217578/point-in-polygon-aka-hit-test
	//http://www.ecse.rpi.edu/Homepages/wrf/Research/Short_Notes/pnpoly.html
	var x = point.x;
	var y = point.y;
	var p = polygon;
	var n = p.length;
	var res = false;
	for (var i=0, j=n-1; i < n; j = i++)
	{
		//if p[i].y and p[j].y not on same side of y
		//and
		//x <
		if ( ((p[i].y>y) != (p[j].y>y)) &&
			x < p[i].x + (p[j].x-p[i].x) * (y-p[i].y) / (p[j].y-p[i].y) )
		{
			res = !res;
		}
	}
	return res;
}


/**
 * Clip a polygon with a convex (!) polygon
 * Polygon is defined as array of arrays (points as [x,y] arrays)
 * Polys have to be defined clockwise (with inside on the right)
	var subjectPolygon = [[50, 150], [200, 50], [350, 150], [350, 300], [250, 300], [200, 250], [150, 350], [100, 250], [100, 200]],
	var clipPolygon = [[100, 100], [300, 100], [300, 300], [100, 300]];
	var clippedPolygon = GeomUtil.clipPolygon(subjectPolygon, clipPolygon);
 **/
GeomUtil.clipPolygon = function(subjectPolygon, clipPolygon)
{
	//http://en.wikipedia.org/wiki/Sutherland-Hodgman_algorithm
	//http://rosettacode.org/wiki/Sutherland-Hodgman_polygon_clipping
	//http://jsfiddle.net/elisherer/y6RDB/
	var cp1, cp2, s, e;
	var inside = function (p)
	{
		return (cp2[0]-cp1[0])*(p[1]-cp1[1]) > (cp2[1]-cp1[1])*(p[0]-cp1[0]);
	}
	var intersection = function ()
	{
		var dc = [ cp1[0] - cp2[0], cp1[1] - cp2[1] ],
				dp = [ s[0] - e[0], s[1] - e[1] ],
				n1 = cp1[0] * cp2[1] - cp1[1] * cp2[0],
				n2 = s[0] * e[1] - s[1] * e[0],
				n3 = 1.0 / (dc[0] * dp[1] - dc[1] * dp[0]);
		return [(n1*dp[0] - n2*dc[0]) * n3, (n1*dp[1] - n2*dc[1]) * n3];
	}
	var outputList = subjectPolygon;
	cp1 = clipPolygon[clipPolygon.length-1];
	for (var j in clipPolygon)
	{
		var cp2 = clipPolygon[j];
		var inputList = outputList;
		outputList = [];
		s = inputList[inputList.length - 1];
		for (var i in inputList)
		{
			var e = inputList[i];
			if (inside(e))
			{
				if (!inside(s)) outputList.push(intersection());
				outputList.push(e);
			}
			else if (inside(s))
			{
				outputList.push(intersection());
			}
			s = e;
		}
		cp1 = cp2;
	}
	return outputList;
}