(function(window) {
var Scene3D = function(width, height, focalLength, lightVector) {
	this.initialize(width, height, focalLength, lightVector);
}

var p = Scene3D.prototype = new Container();

p.initialize = function(width, height, focalLength, lightVector)
{
	Container.prototype.initialize.apply(this);
	this.matrix = mat4.identity(mat4.create());
	this.scale = 1;
	this.focalLength = focalLength;
	//topview:
	this.rotationX = -90/180*Math.PI;
	//
	this.width = width;
	this.height = height;
	this.mode = Mode.NORMAL;
	//set camera pos and angle
	mat4.scale(this.matrix, this.matrix, vec3.fromValues(this.scale, this.scale, this.scale));
	mat4.rotateX(this.matrix, this.matrix, this.rotationX);
	//
	this.groundShadowsOn = Config.groundShadowColor!=Config.groundColor;
	//
	this.shader = new Shader(lightVector);
	//some reusable objects used in rendering
	this.v3 = vec3.create();
	this.v2 = vec2.create();
	//
	this.objects = [];
	this.objectsVisible = [];//unculled objects of last render
	this.reset();
}

//public
p.setSize = function(width, height)
{
	this.width = width;
	this.height = height;
}

//public
p.clear = function()
{
	this.objects.length = 0;
	this.removeAllChildren();
}

//public
p.reset = function(width, height)
{
	this.clear();
	this.shader.reset();
	this.layerGround = new Shape();
	this.layer0 = new Shape();
	this.layer1 = new Shape();
	this.layerInfo = new Container();
	this.addChild(this.layerGround);
	this.addChild(this.layer0);
	this.addChild(this.layer1);
	this.addChild(this.layerInfo);
	mat4.setXYZ(this.matrix,0,0,0);
}

//public
p.addObject = function(obj)
{
	//obj is Cube3D
	this.objects.push(obj);
	this.shader.addObject(obj);
}
//public
p.removeObject = function(obj)
{
	//obj is Cube3D
	var idx = this.objects.indexOf(obj);
	//log("Scene3D.removeObject",obj.id,idx);
	if (idx>-1) this.objects.splice(idx,1);
	this.shader.removeObject(obj);
}
//public
p.updateObject = function(obj)
{
	this.shader.updateObject(obj);
}

//public
p.setMode = function(value)
{
	if (this.mode==value) return;
	this.mode = value;
}

//public
p.render = function(viewport)
{
	//viewport.expand(-200,-200);//TEST
	var p2 = this.v2;//reusable object for 2d point
	for (var i=0;i<this.objects.length;i++)
	{
		this.objects[i].applySceneTransform(this.matrix);
	}
	//check frustum culling, which is kept simple (works for topview only):
	//	calc visible part of floor (=slice of frustum at y=0)
	//    this is same as 2d viewport because of top view (but compare viewport.y to cube.z) ! :
	//		floor: y=0 becomes z'=0 because of top view, so scale = focalLength/(z+focalLength) = 1.
	//	and check if cube bottoms are in this rect
	var objs = this.objectsVisible;
	objs.length = 0;
	for (var i=0;i<this.objects.length;i++)
	{
		var o = this.objects[i];
		var r = this.shader.getGroundShadowRectForObject(o);
		if (viewport.intersects(r)) objs.push(o);
	}
	objs.sort(this.compareZOrder);
	//
	this.layerInfo.removeAllChildren();
	this.layerGround.graphics.clear();
	this.layer0.graphics.clear();
	this.layer1.graphics.clear();
	if (!this.showFloorPlan)
	{
		var normal = this.mode==Mode.NORMAL;
		if (normal && this.groundShadowsOn)
		{
			//draw ground shadows
			var g = this.layerGround.graphics;
			for (var i=0;i<objs.length;i++)
			{
				var ps = this.shader.getGroundShadowForObject(objs[i]);
				if (!ps) continue;
				var len = ps.length;
				g.beginFill(Config.groundShadowColor);
				g.setStrokeStyle(.5).beginStroke(Config.groundShadowColor);
				this.to2d(ps[len-1], p2);
				g.moveTo(p2[0],p2[1]);
				for (var k=0;k<len;k++)
				{
					this.to2d(ps[k], p2);
					g.lineTo(p2[0],p2[1]);
				}
				g.endFill();
			}
		}
		//
		var faceIdx = normal ? 0 : 1;
		var g = this.layer0.graphics;
		for (var i=0;i<objs.length;i++)
		{
			var o = objs[i];
			//first draw all faces, then draw shadows on faces
			//draw faces
			for (var j=faceIdx;j<5;j++)
			{
				var face = o.getFace(j, !normal);
				if (face)
				{
					if (normal)
					{
						//NB: use of color vs colorShadow determined by face index!
						//This is hardcoded and does not use lightvector!
						var color = j<3 ? o.color : o.colorShadow;
						g.beginFill(color);
						g.setStrokeStyle(.5).beginStroke(color);
					}
					else
					{
						g.setStrokeStyle(1).beginStroke(o.color);
					}
					ps = face.points;
					len = ps.length;
					this.to2d(ps[len-1], p2);
					g.moveTo(p2[0],p2[1]);
					//skip topline if not normal (last line of face is topline by design)
					for (var k=0;k<len-(1-normal);k++)
					{
						this.to2d(ps[k], p2);
						g.lineTo(p2[0],p2[1]);
					}
				}
			}
			//draw shadows on objects
			if (normal)
			{
				g.beginFill(o.colorShadow);
				g.setStrokeStyle(1.5).beginStroke(o.colorShadow);
				for (var j=0;j<5;j++)
				{
					if (o.getFace(j))
					{
						//draw shadows on face
						var shadows = this.shader.getShadowsForObjectFace(o,j);
						if (shadows)
						{
							for (var k=0;k<shadows.length;k++)
							{
								var ps = shadows[k];
								this.to2d(ps[0], p2);
								g.moveTo(p2[0],p2[1]);
								for (var n=1;n<ps.length;n++)
								{
									this.to2d(ps[n], p2);
									g.lineTo(p2[0],p2[1]);
								}
							}
						}
					}
				}
			}
		}
	}
	//draw grid & tripod
	var g = this.layer0.graphics;
	if (Config.showGrid)
	{
		//var v3 = vec3.positionFromMat4(vec3.create(), this.matrix);//scene origin
		var origin = vec2.create();
		var r = 100;
		this.to2d( vec3.fromValues(0,0,0), origin);
		this.to2d( vec3.fromValues(r,0,0), p2);
		g.setStrokeStyle(2).beginStroke("#FF0000").moveTo(origin[0],origin[1]).lineTo(p2[0],p2[1]);
		this.to2d( vec3.fromValues(0,r,0), p2);
		g.setStrokeStyle(2).beginStroke("#00FF00").moveTo(origin[0],origin[1]).lineTo(p2[0],p2[1]);
		this.to2d( vec3.fromValues(0,0,r), p2);
		g.setStrokeStyle(2).beginStroke("#0000FF").moveTo(origin[0],origin[1]).lineTo(p2[0],p2[1]);
		//draw camera crosshair
		g.setStrokeStyle(2).beginStroke("#000000").moveTo(-30,0).lineTo(30,0).moveTo(0,-30).lineTo(0,30);
		//draw grid
		//TODO/OPTIMIZE: draw grid once on separate layer and only correct offset
		//   or draw only visible part of grid.
		var g = this.layerGround.graphics;
		var r = 2000;
		var d;
		for (d=-r;d<r;d+=50)
		{
			var b = (d%100!=0) == normal;
			this.to2d( vec3.fromValues(d,0,-r), origin);
			this.to2d( vec3.fromValues(d,0,r), p2);
			g.setStrokeStyle(b?2:.5).beginStroke(b?"#808080":"#BBBBBB").moveTo(origin[0],origin[1]).lineTo(p2[0],p2[1]);
			this.to2d( vec3.fromValues(-r,0,d), origin);
			this.to2d( vec3.fromValues(r,0,d), p2);
			g.setStrokeStyle(b?2:.5).beginStroke(b?"#808080":"#BBBBBB").moveTo(origin[0],origin[1]).lineTo(p2[0],p2[1]);
		}
	}
	//
	if (this.showFloorPlan)
	{
		//show floorplan + info
		var p3 = this.v3;
		for (var i=0;i<objs.length;i++)
		{
			var o = objs[i];
			var face = o.getFace(Face.T);
			var text = new Text(o.data, "12px Arial", "#000000");
			vec3.copy(p3, face.points[3]);
			p3[1] = 1;
			this.to2d(p3, p2);
			text.x = p2[0]+5;
			text.y = p2[1]+5;
			this.layerInfo.addChild(text);
			//show ground plane
			g.beginFill(face.color);
			g.setStrokeStyle(1).beginStroke("#000000");
						
			ps = face.points;
			len = ps.length;
			vec3.copy(p3, ps[len-1]);
			p3[1] = 1;
			this.to2d(p3, p2);
			g.moveTo(p2[0],p2[1]);
			var y = ps[len-1][1];
			for (var k=0;k<len;k++)
			{
				vec3.copy(p3, ps[k]);
				p3[1] = 1;
				this.to2d(p3, p2);
				g.lineTo(p2[0],p2[1]);
			}
		}
	}
}

p.compareZOrder = function(o0,o1)
{
	return o0.dist<o1.dist ? 1 : -1;
	//if overlap in x, compare z distance to cam
	if (o0.xMin<=o1.xMax && o1.xMin<=o0.xMax)
	{
		return o0.distZ<o1.distZ ? 1 : -1;
	}
	//if overlap in z, compare x distance to cam
	if (o0.zMin<=o1.zMax && o1.zMin<=o0.zMax)
	{
		return o0.distX<o1.distX ? 1 : -1;
	}
	//else compare dist of centers to cam
	else return 0 ;//
}

p.to2d = function(p3d, p2d)
{
	var v3 = this.v3;
	vec3.transformMat4(v3, p3d, this.matrix);
	var s = this.focalLength/(v3[2]+this.focalLength);
	p2d[0] = v3[0] * s;
	p2d[1] = v3[1] * s;
}

window.Scene3D = Scene3D;
}(window));