/**
 * Shared code for rr pieces
 * 20140115 : added window.isMobile
 * 20131007 : inlined device check
 * 20130711 : added getQueryParam
 * 20130531 : added log2
 **/

var piece;
var canvas;
var isMobile = /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent);

function init(canvasId, mainClass, config)
{
	canvas = document.getElementById(canvasId);
	piece = new mainClass(canvas, config);
	//if no parent, we are not in an iframe so we handle resizing ourselves
	if (!window.parent || window.parent.onResize==null)
	{
		window.onresize = function() { onWindowResize(); };
		window.onorientationchange = function() { onWindowResize(); };
	}
	onWindowResize();
	piece.start();
}

function onWindowResize()
{
	setSize(window.innerWidth, window.innerHeight);
}
function setSize(w,h)
{
	//can also be called from parent window
	var s = window.devicePixelRatio;
	if (isNaN(s) || s==0) s = 1;
	canvas.getContext('2d').scale(s,s);
	canvas.width = w*s;
	canvas.height = h*s;
	//
	if (isMobile)
	{
		//bound on 2046 to avoid ios performance issue when canvas is larger!
		//  see: http://www.scirra.com/forum/retina-ios-performance-problem-fix-please-test_topic58742.html
		canvas.width = Math.min(2046, canvas.width);
		canvas.height = Math.min(2046, canvas.height);
	}
	canvas.style.width = w+"px";
	canvas.style.height = h+"px";
	//log("setSize",w,h,s,w*s,h*s);
	if (piece) piece.setSize(w,h,s);
}

function log()
{
	if (window.console && window.console.log) window.console.log(arguments);
	log2.apply(null, arguments);
}
function log2()
{
	if (document.getElementById("output"))
	{
		var str = "";
		for (var i=0;i<arguments.length;i++)
		{
			if (i>0) str += ", ";
			str += arguments[i];
		}
		//document.getElementById("output").innerHTML = str + "<br/>" + document.getElementById("output").innerHTML;
		document.getElementById("output").innerHTML = str;
	}
}

function getQueryParam(key)
{
	key = key.replace(/[\[]/,"\\\[").replace(/[\]]/,"\\\]");
	var regex = new RegExp("[\\?&]"+key+"=([^&#]*)");
	var qs = regex.exec(window.location.href);
	return qs == null ? qs : qs[1];
}