/**
 * Constructor: create Plane3d from a point and a normal vector
 */	
var Plane3d = function(point3d, normalVector) 
{
	this.p = point3d;
	this.n = normalVector.normalize();//NB: unit normal vector!
	//parameterization of plane : ax+by+cz+d = 0 (for any point p1={x,y,z} on plane)
	//where {a,b,c} = this.n, so  ax+by+cz = n dot p1
	//and d = - n dot p
	//so for any point p1 on plane: n dot p1 - n dot p = 0;
	this.d = - this.n.dot(this.p);
	//because this.n is unit normal, d is distance to origin
	//http://mathworld.wolfram.com/Plane.html
}

/**
 * Factory method: create a Plane3d from 3 non-linear point3ds
 */	
Plane3d.fromPoints = function(p0,p1,p2)
{
	//calc normal and define plane as point3d and normal vector
	var v0 = p1.subtract(p0);
	var v1 = p2.subtract(p0);
	var n = v0.cross(v1);
	return new Plane3d(p0, n);
}


Plane3d.prototype.getDistanceToPoint	= function(p)
{
	return this.n.dot(p) + this.d;
}
	
/**
 * Calc intersection with line
 * p0 and p1 describe a line (from p0 to p1)
 */
Plane3d.prototype.getIntersectionWithLine = function(p0,p1)
{
	var v = p1.subtract(p0);//vector from p0 to p1
	var nDotP0 = this.n.dot(p0);
	var nDotV = this.n.dot(v);
	return p0.add( v.multiply((-this.d - nDotP0)/nDotV) );	
}
/**
 * Calc intersection with line
 * p and v describe a line (where p is point and v is vector)
 */
Plane3d.prototype.getIntersectionWithLineVector = function(p0,v)
{
	var nDotP0 = this.n.dot(p0);
	var nDotV = this.n.dot(v);
	return p0.add( v.multiply((-this.d - nDotP0)/nDotV) );	
}

/**
 * Calc intersection with plane
 */
Plane3d.prototype.getIntersection = function(P)
{
	//http://mathworld.wolfram.com/Plane-PlaneIntersection.html
}